#!/usr/bin/perl -w
#BWAPIPELINE1_run_bwa.pl
#### downloads a metagenomic short read file for a metahit sample and runs BWA to align it to a set of reference genomes (contigs)
#### outputs a .bam file with alignments
###################################################

use lib '/net/gs/vol1/home/greensi/perl/General';
require "download_file_from_FTP.pl";
require "MergeFiles.pl";

####################################################
## FILES AND DIRECTORIES
#################################################
$LAB_PATH='/net/borenstein/vol1';
$HOME = '/net/gs/vol1/home/greensi';
$SAMPLE_DIR= $LAB_PATH . '/DATA_RAW/METAHIT/METAHIT_HUMAN_GENE_CATALOG/SAMPLES_SHORT_READ_DATA/';
$RefGenomes = $HOME . '/Projects/SG_SINGLEGENOMES/DATA/landscape/genomeFasta/LANDSCAPE.refGenomes.merged_genomes.fasta';
$BWA_OUTPUT_DIR = $LAB_PATH . '/PROJECTS/StrainVariation_SG/BWA/';
$| = 1;

##########################################
### READ IN FASTQ ACCESSIONS FOR A SAMPLE
##########################################
$infile = pop (@ARGV);
open(INFILE, "$infile") or die "Kozyol cannot read $infile!\n";
($sample,$rest) = split(/\t/,<INFILE>);
print "$sample:";
@to_download = split(/\|/,$rest);
@fastqs=();
foreach $file (@to_download)
{ 	
	($acc) = ($file =~ m/\/(.+)_1.fastq.bz2/);
	print "\n--$acc..."; 
	$saveas_file = $SAMPLE_DIR . $sample . ".$acc" . ".fastq.bz2"; 
	print "save as: $saveas_file\n";
	##########################################
	### DOWNLOAD FILE FROM FTP
	if (not -e $saveas_file)
	{	print "downloading $file...";
		&download_from_ftp('ftp.ddbj.nig.ac.jp/ddbj_database/dra/fastq/ERA000/ERA000116/',$file,$saveas_file);
	}
	else
	{	print "$file already downloaded...";	}
	##########################################
	### UNZIP FILE
	print "unzipping...";
	system('bunzip2', $saveas_file);
	if ($saveas_file =~ m/bz2/)
	{	$saveas_file =~ s/.bz2//;	} 
	push @fastqs, $saveas_file;
}

##########################################
## MERGE ALL FASTQs FOR A SAMPLE
print "merging...";
&merge_files(@fastqs,"$sample.merged.fastq");

##########################################
### REZIP INDIVIDUAL FASTQs
print "re-zipping files...\n";
foreach $file (@fastqs)
{	system('bzip2', $file);	print "$file\n";}

##########################################
### ALIGN MERGED FASTQ TO REFGENOMES WITH BWA
print "aligning...\n";
`perl perl/General/run_bwa.pl $sample.merged.fastq $RefGenomes 5 75`;
`cat $sample.merged.fastq.bam > $BWA_OUTPUT_DIR$sample.merged.bam`;

##$HOME/Programs/bwa-0.6.2/bwa aln -n 3 -t 4 $RefGenomes $sample.merged.fastq \> $BWA_OUTPUT_DIR$sample.merged.sai`;
##$HOME/Programs/bwa-0.6.2/bwa samse -n 100 $RefGenomes $BWA_OUTPUT_DIR$sample.merged.sai $sample.merged.fastq \> $BWA_OUTPUT_DIR$sample.merged.sam`;

##########################################
### ERASE THE MERGED FASTQ
system('rm','-f', "$sample.merged.fastq");

print "file output to $BWA_OUTPUT_DIR$sample.merged.bam\n";
##########################################
### CONVERT THE SAM FILE TO A BAM FILE
#`$HOME/Programs/samtools-0.1.18/samtools view -bS $BWA_OUTPUT_DIR$sample.merged.sam \> $BWA_OUTPUT_DIR$sample.merged.bam`;

##########################################
### REMOVE THE SAM FILE
#system('rm','-f', "$BWA_OUTPUT_DIR$sample.merged.sam");


die;