#!/usr/bin/perl -w
###################################################
#$koCov_outfile = shift(@ARGV);

############################################################
### FILES
############################################################
	$f_genomeClusters = 'Projects/SG_SINGLEGENOMES/DATA/genomeInfo/LANDSCAPE.refGenomes.cluster';
	$f_geneAnn = 'Projects/SG_SINGLEGENOMES/DATA/geneAnnotations/LANDSCAPE.refGenomes.merged_proteins.annotated';
	$f_genomeFilters = 'Projects/SG_SINGLEGENOMES/DATA/testBWA/filterGenomes.txt';
	$f_koFilters = 'Projects/SG_SINGLEGENOMES/DATA/testBWA/filterKOs.txt';

############################################################
### VARIABLES
############################################################
%h_readCts = (
	mapped => 0,
	geneMapped => 0,
	koMapped => 0,
	dup => 0,
);

%h_genes=();
%h_kos=();
%h_clusters=();

%h_gene2KO=();
%h_taxID2cluster = ();
%h_cluster2taxID = ();

%h_koCov=();

$qID = ""; @qHits=(); $nof_qHits=0; $hitStatus="";

############################################################
### READ IN CLUSTERS AND KOS
############################################################
getKOs();
getClusters();
%h_kos=();
%h_clusters=();

############################################################
### READ IN AND COUNT ENTRIES
############################################################

while (<>) {

	#1)get read tax
	#2)get read clust
	#3)get read gene
	#4)get read KO - unannotated?
	#5)other genomes in clust?
	#6)other clust-genomes with KO?
	#7)get hit tax/clust/KO - match?
	
	# dont print headers
	$line=$_; chomp($line); 
	if (/@/)
	{	next;	}
	
	@fields = split(/\t/,$line);
	
	if($fields[3] =~ /^dup/)
	{ $h_readCts{"dup"}++; next:	}
	
	if($fields[3] ne $qID)
	{	
		#############################
		# print last query
		##############################
		printLastQuery();
		@qHits=(); $nof_qHits=0;
		$h_readCts{$hitStatus}++;
		
		#############################
		# get new QUERY info
		##############################
		$qID = $fields[3];
		$hitStatus="mapped";
	}
		
	#############################
	# get HIT cluster info
	##############################	
	$hContig = $fields[0];
	($hTaxID) = ($hContig =~ /^(\d+)_/);
	if (not exists $h_taxID2cluster{$hTaxID})
	{	next;	}
	$hClust = $h_taxID2cluster{$hTaxID};
	
	#############################
	# get HIT KO info
	##############################	
	$overlap = $fields[21];
	$hLength = $fields[14]-$fields[13];
	if ($overlap > 0)
	{	
		$hitStatus="geneMapped";
		$hGeneID = $fields[15];
		$h_genes{$hGeneID}=1;
		$hGeneCov = $overlap/$hLength;
		$hKO = $h_gene2KO{$hGeneID};
		if (defined $hKO)
		{	
			$nof_qHits++;
			@hKOs = split(/\|/,$hKO);
			foreach $ko (@hKOs)
			{	
				($koID,$koweight)=split(/:/,$ko);	
				$koCov = $koweight*$hGeneCov;
				push (@qHits, "$hClust,$koID,$koCov");	
			}
		}
	}
}	
printLastQuery();
$h_readCts{$hitStatus}++;

#############################
# print totals
##############################
foreach $type (sort keys %h_readCts)
{	print "$type:$h_readCts{$type},";	}
print "KCs:" . scalar(keys %h_koCov) . ",KOs:" . scalar(keys %h_kos) . ",Clusters:" . scalar(keys %h_clusters);

die;

############################################################
### SUBROUTINES
############################################################
sub getKOs
{	### READ IN GENE ANNOTATIONS
	
	%h_kFilter=();
	open (FH_KF, "$f_koFilters") or die "Cannot open $f_koFilters for reading.\n";
	while (<FH_KF>)
	{	
		$line = $_; chomp($line);
		$h_kFilter{$line}=1;
	}
	close(FH_KF);
	
	$nof_genes = 0;
	$nof_kos = 0;
	open (FH_ANN, "$f_geneAnn") or die "Cannot open $f_geneAnn for reading.\n";
	while (<FH_ANN>)
	{	
		$line = $_; chomp($line);
		($entry,$ko,$weight)= split(/\t/,$line);
		if (defined $ko)
		{	
			if (not exists $h_kFilter{"K$ko"})
			{	
				($geneID,$rest) = split(/_/,$entry);  $rest=1;
				if (exists 	$h_gene2KO{$geneID})
				{	$h_gene2KO{$geneID} .= "K$ko:$weight|" ;	}
				else
				{	$h_gene2KO{$geneID} = "K$ko:$weight|" ;	$nof_genes++;}	
				if (! exists $h_kos{"K$ko"})
				{	$h_kos{"K$ko"}=1;	
					$nof_kos++;
				}
				($taxID) = ($geneID =~ /G(\d+)/);
				$h_taxID2kos{$taxID} .= "K$ko|"
			}
		}
	}
	close(FH_ANN);
	return 0;
}

sub getClusters
{	### READ IN GENOME CLUSTERS
	%h_gFilter=();
	open (FH_GF, "$f_genomeFilters") or die "Cannot open $f_genomeFilters for reading.\n";
	while (<FH_GF>)
	{	
		$line = $_; chomp($line);
		$h_gFilter{$line}=1;
	}
	close(FH_GF);
	
	$nof_clusters=0;
	$nof_taxIDs = 0;
	open (FH_CLUSTERS, "$f_genomeClusters") or die "Cannot open $f_genomeClusters for reading.\n";
	$line = <FH_CLUSTERS>;
	while (<FH_CLUSTERS>)
	{	$nof_taxIDs++;
		$line = $_; chomp($line);
		($taxID,$cluster) = split(/\t/,$line);
		if (exists $h_gFilter{$taxID})
		{	next;	}
		$h_taxID2cluster{$taxID}=$cluster;	
		if (! exists $h_cluster2taxID{$cluster})
		{	$h_cluster2taxID{$cluster}="$taxID|";	
			$nof_clusters++;
		}
		else
		{	$h_cluster2taxID{$cluster} .= "$taxID|";	}
		
	}
	close(FH_CLUSTERS);
	return 0;
}

sub printLastQuery
{	
		if ($nof_qHits>0)
		{
			$hitStatus="koMapped";
			foreach $hit (@qHits)
			{	
				($clust,$ko,$cov) = split(/,/,$hit);
				$h_koCov{$clust . "_" . $ko} += $cov/$nof_qHits;	
				$h_kos{$ko}=1;
				$h_clusters{$clust}=1;
			}
			
		}
	return 0;
}