#!/usr/bin/perl -w

## merges genes from individual fastas into one multi-entry file, according to a supplied list of taxon iDs for a specific project

### read in a paired list of the form [DB,taxID_file], where 'DB'is the name of the folder in U:\RefSets\LANDSCAPE_REF_GENOMES\ corresponding to a specific database, and taxID_file is a file listing the taxonIDs desired from that DB
### for each taxon, in a DB, 
### unzip corresponding file(keep original) and open each fasta (may be just 1 or >1),
### make sure its not a plasmid
### open corresponding gene file
### extract each gene sequence and create header: [geneID_contigID_start_stop; optional: geneName, geneLength]
### add sequence to merged file
### remove each
$dir_calls = '/net/gs/vol1/home/greensi/RefSets/LANDSCAPE_REF_GENOMES/NCBI_ALL/rna_calls/';
$ext = 'rnt';
$geneIdentifier="R";#"G";
$f_out = $dir_calls . 'LANDSCAPE.refGenomes.geneInfo_multStrainClusters.n137.BED';
$f_fastaHeaders = '/net/gs/vol1/home/greensi/Projects/SG_SINGLEGENOMES/DATA/landscape/genomeFasta/LANDSCAPE.refGenomes.merged_genomes.fasta.headers';
$f_taxIDs = '/net/gs/vol1/home/greensi/Projects/SG_SINGLEGENOMES/DATA/landscape/taxID_lists/LANDSCAPE.refGenomes.taxID_multStrainClusters.n137.txt';
#$f_taxIDs = shift(@ARGV);
%h_contig2header=();
%h_taxID2genes=();

################# READ IN FASTA HEADERS##########################
print "reading in fasta headers ... $f_fastaHeaders \n";
	open (FH, "$f_fastaHeaders") or die "Cannot open $f_fastaHeaders for reading.\n";
	while(<FH>) {
	#while (<>){
		$line=$_; chomp($line); 
		@fields=split(/ /,$line);
	  ($header) = ($fields[0] =~ m/>(.+)/); 
		($taxID,$contig) = ($header =~ m/([^_]+)_\d+_(.+)$/);  
		$h_contig2header{$contig}=$header;
	} 
	close(FH);
	
	

############### READ IN DESIRED TAX IDS################################
print "reading in tax ids.... $f_taxIDs\n";	
open (FH2, "$f_taxIDs") or die "Cannot open $f_taxIDs for reading.\n";
	while(<FH2>) {
		$taxID=$_; chomp($taxID); 
		$h_taxID2genes{$taxID}=0; 
	}
	close(FH2);

############## SET UP OUT FILES##########################################
	open (LOG, ">$f_out.log") or die "Cannot open logfile for writing.\n";
	select((select(LOG), $|=1)[0]);
	open (FH_BED, ">$f_out") or die "Cannot open BEDfile for writing.\n";
	select((select(FH_BED), $|=1)[0]);

##################READ IN CALL FILES#######################################
	print "reading in call files...$dir_calls\n";
	# --- --- Get a list of files in dir --- --- #
	$nof_taxIDs = 0; $nofGenes = 0;
	@files = glob("$dir_calls*.$ext $dir_calls*.$ext.tgz" ); 

	foreach $file (@files)
		{ 	
			## get taxID 
			($taxID) = ($file =~ m/\/([^\/_]+)_[^\/]+.rnt/);
			$h_taxID2genes{$taxID}++; 
		} 
	 
	foreach $file (@files)
		{ 	
			## get taxID 
			#print "$file\n";
			($taxID) = ($file =~ m/\/([^\/_]+)_[^\/]+.rnt/);
			
			## copy or unzip each to 'unzipped' folder
			if ($file =~ m/.tgz/)
			{	system('tar', '-xzf', "$file", '-C',$dir_calls . 'unzipped');	}
			else
			{	system('cp', $file, $dir_calls . 'unzipped');	}			
			
			## open each rnt record, get contig name and get first line
			@calls = glob("$dir_calls/unzipped/*.$ext" );
			foreach $call_file (@calls)
			{
				($contig)= ($call_file =~ m/([^\/\_\d][^\/]*).$ext/);
				open (CALLS, $call_file) or die "cannot open $rnt";
				$line=<CALLS>;	#print "$line\n";
				
				## look up full header from contig name
				if (exists $h_contig2header{$contig})
				{	$header = $h_contig2header{$contig};	
					($taxID,$contig) = ($header =~ m/([^_]+)_\d+_(.+)$/);
					#print "----$header\n"; 
					
					$line=<CALLS>; chomp($line);
					($nof_orfs) = ($line =~ m/^(\S+)/);
					<CALLS>;
					while(<CALLS>)
					{	
						$line=$_; chomp($line);
						($loc, $strand, $length, $pid, $gene, $synonym, $code, $cog, $product) = split(/\t/,$line);
						($start,$end) = ($loc =~ m/(.+)\.\.(.+)/);
						$h_taxID2genes{$taxID}++; $nofGenes++;
						$myGeneID = $geneIdentifier . $taxID . "-" . $h_taxID2genes{$taxID};
						print FH_BED "$header\t$start\t$end\t$myGeneID\t$length\t$strand\t$pid\t$gene\t$synonym\t$code\t$product\n";
					}
				}
				else
				{	print "-----cant find: $taxID\_$contig: $line\n";}
				
				
				close(CALLS);	 
				system('rm',$call_file);
			}
		}
		close(FH_BED);	
		
	
## PRINT COUNTS
	print LOG "************************************\n";
	print LOG "$nofGenes total genes merged! \n";
	print LOG "nof genes merged for each taxID:\n";
	print  "************************************\n";
	print  "$nofGenes total genes merged! \n";
	print  "nof genes merged for each taxID:\n";
	foreach $taxID (sort keys %h_taxID2genes)
	{	print LOG "$taxID:$h_taxID2genes{$taxID}\n";	
		print "$taxID:$h_taxID2genes{$taxID}|";	
	}
	print LOG "************************************\n";
	print LOG "\n the following taxIDs had no valid contigs:\n";
	print  "************************************\n";
	print "\n the following taxIDs had no valid contigs:\n";

	foreach $taxID (sort keys %h_taxID2genes)
	{	if ($h_taxID2genes{$taxID}<1)
		{	print LOG "$taxID|";
			print "$taxID|";
		}
	}
	print LOG "\n";
	print "\n";


	
close(LOG);			