#!/usr/bin/perl -w

### convert rogan's gene annotation files to matrix of genomes v KOs, with summed gene weights
$num_args = $#ARGV + 1;
if ($num_args < 2) 
{
  print "\nUsage: LANDSCAPE6_parse_gene_annotations.pl outfile annotations_file1 \[annotations_file2\] \n";
  exit;
}
$f_outfile = shift @ARGV;

$nof_unannotated_genes=0;
$nof_annotated_genes = 0;
%h_taxID2annCt = ();
%h_taxID2allCt = ();
%h_ko2Ct = ();
%hoh_taxID2koCts = ();
%empty_hash=();
$weightThresh=0;


@skip_genomes = ();
%change_genomes = ();

#@skip_genomes = ("G391904-.+_C1", "G469008-.+_C1", "G525362-.+_C([8-9]|[1-9][0-9])", "G655813-.+_C(1[1-9]|[2-9][0-9])");
#%change_genomes = ("G391904-.+_C2" => 2552, "G469008-.+_C2" => 4208);
$nof_skipped = 0; $nof_changed = 0;
##example entry(unannotated): #G411459-3095_C50_237349_237996
##example entry(annotated): G398513-401_C1_494969_495250	02221	1
while (@ARGV)
{
$f_infile = shift @ARGV;
open (FH_IN, "$f_infile") or die "Cannot open $f_infile for reading.\n";
while (<FH_IN>)
{	
	$line = $_; chomp($line);
	$skip = 0;
	foreach $skip_entry (@skip_genomes)
	{	if ($line =~ m/$skip_entry/)
		{	$skip=1;	$nof_skipped++; }
	}
	foreach $change_entry (keys %change_genomes)
	{
		if ($line =~ m/$change_entry/)
		{	
			($geneID,$contigID,@rest) = split(/_/,$line);
			($taxID,$geneNum) = split(/-/,$geneID); 
			$newGeneNum = $geneNum - $change_genomes{$change_entry}; 
			$line = $taxID . "-" . $newGeneNum . "_C1_";
			while (@rest)
			{	$line .= shift(@rest);	
				if (@rest)
				{ $line .= "_";	}
			} 
			$nof_changed++;
		}
	}
	if (!$skip)
	{
	($taxID) = ($line =~ m/^\#*G(.+)\-/);
	if (defined $taxID)  ## if its an actual entry
	{
		if (! exists $h_taxID2allCt{$taxID})  ##if its a new taxID
		{	
			$h_taxID2allCt{$taxID}=0;
			$h_taxID2annCt{$taxID}=0;
			#$hoh_taxID2koCts{$taxID}=%empty_hash;
		}
	
		if (substr($line,0,1) eq "#")  ## if its unannotated
		{	$nof_unannotated_genes++;	
			$h_taxID2allCt{$taxID}++;
		}
		
		else
		{
			($geneID,$ko,$weight)= split(/\t/,$line);
			if ($weight>=$weightThresh)
			{
			$nof_annotated_genes++;
			$h_taxID2annCt{$taxID} += $weight;	## add to taxon counts
			$h_taxID2allCt{$taxID} += $weight;	
			
			if (! exists $h_ko2Ct{$ko})	## if its a new ko
			{	$h_ko2Ct{$ko}=$weight;	}
			else
			{	$h_ko2Ct{$ko}+= $weight;	}
			
			if (! exists $hoh_taxID2koCts{$taxID}{$ko})
			{	$hoh_taxID2koCts{$taxID}{$ko}=$weight;	}
			else
			{	$hoh_taxID2koCts{$taxID}{$ko}+= $weight;	}
			}
		}
	}
	else
	{	print "$line\n";	}
	}
}	
close(FH_IN);
}

print "$nof_annotated_genes annotated genes\n";
print "$nof_unannotated_genes unannotated genes\n";
print "$nof_changed changed genes\n";
print "$nof_skipped skipped genes\n";

open (FH_OUT_TAXIDS, ">$f_outfile.taxIDs") or die "Cannot open $f_outfile for writing.\n";
open (FH_OUT_KOS, ">$f_outfile.kos") or die "Cannot open $f_outfile for writing.\n";
open (FH_OUT_KOLIST, ">$f_outfile.kolist") or die "Cannot open $f_outfile for writing.\n";

$nof_taxIDs=0;
foreach $taxID (sort keys %h_taxID2allCt)
{	$fraction_annotated=$h_taxID2annCt{$taxID} / $h_taxID2allCt{$taxID};
	$nof_taxID_kos = scalar(keys %{$hoh_taxID2koCts{$taxID}});
	print FH_OUT_TAXIDS "$taxID\t$h_taxID2annCt{$taxID}\t$h_taxID2allCt{$taxID}\t$fraction_annotated\t$nof_taxID_kos\n";	$nof_taxIDs++;
}
print "$nof_taxIDs taxIDs\n";

$nof_kos=0;
foreach $ko (sort keys %h_ko2Ct)
{	print FH_OUT_KOS "$ko\t$h_ko2Ct{$ko}\n";	$nof_kos++;}
print "$nof_kos kos\n";

foreach $taxID (sort keys %h_taxID2allCt)
{
	print FH_OUT_KOLIST "$taxID)";
	foreach $ko (sort keys %h_ko2Ct)
	{
		if (exists $hoh_taxID2koCts{$taxID}{$ko})
		{	print FH_OUT_KOLIST "K$ko:$hoh_taxID2koCts{$taxID}{$ko}|";	}
	}
	print FH_OUT_KOLIST "\n";
}

close(FH_OUT_TAXIDS);
close(FH_OUT_KOS);
close(FH_OUT_KOLIST);