#!/usr/bin/perl -w

## merges individual fastas into one multi-entry file, according to a supplied list of files for a specific project

### read in a list of taxon IDs
### for each, open list of gene calls
### create header: [geneID_contigID_start_stop_pid; optional: geneName, geneLength]
### save as hash {pid} = header
### read in each protein fasta (.faa), extract pid from header (ie. SARI_00001)
### if its in the hash, print the gene header, followed by the protein seq from the .faa file
### write to log

$num_args = $#ARGV + 1;
if ($num_args < 3)
{
  print "\nUsage: LANDSCAPE_merge_protein_seqs.pl outfile gene_headers DB1 [DB2]\n";
  exit;
}
#############################################
### OPEN FILES
##############################################
$f_merged = shift @ARGV;
$f_headers = shift @ARGV;

open (FH_MERGED, ">$f_merged") or die "Cannot open $f_merged for writing.\n";
open (LOG, ">$f_merged.log") or die "Cannot open logfile for writing.\n";
select((select(LOG), $|=1)[0]);

##############################################
## READ IN NUCELEOTIDE HEADERS, STORE AS HASH OF PIDs
##############################################
$dir_fastas = '/net/gs/vol1/home/greensi/RefSets/LANDSCAPE_REF_GENOMES/';
print "getting records for... \n";
$nof_headers=0;
%h_pid2header=();
%h_pid2count=();
open (FH_HEADERS, "$f_headers") or die "Cannot open headersfile for reading.\n";
while(<FH_HEADERS>) {
	$line=$_; chomp($line);
	($geneID,$length,$pid,$geneName,$geneAnn)=split(/\s/,$line); print "$geneID,$length,$pid,$geneName,$geneAnn\n"; die;
	($prefix,$pid) = split(/:/,$pid);
	$h_pid2header{$pid}=$line;
	$h_pid2count{$pid}=0;
	$nof_headers++;
} print "\n";
close(FH_HEADERS);
print "$nof_headers entries\n";

##############################################
## READ IN PROTEIN RECORDS, OPEN EACH FASTA, CHECK PID AGAINST HASH, AND WRITE TO MERGED FILE AS APPROPRIATE
##############################################
while (@ARGV)
{
	$DB = shift @ARGV;

	# --- --- Get a list of desired species from DB --- --- #
	$nof_entries = 0;
	print "getting records from $DB ... \n";
	@files = glob($dir_fastas . $DB . '/protein_fastas/*.faa.tgz' );
	foreach $file (@files)
	{	#print "$file|";
		if ($DB eq "NCBI_GENBANK")
		{ $nof_comps=4;	}
		else
		{ $nof_comps = 0;	}
		system('tar', '-xzf', "$file", "--strip-components=$nof_comps", '-C',$dir_fastas);
		@fastas = glob($dir_fastas . "*.faa");
		foreach $fasta (@fastas)
		{
			open (FH, $fasta) or die "Cannot open $fasta for reading.\n";
			while (<FH>)
			{
				$line = $_; chomp $line;
				if (substr($line,0,1) eq ">")
				{	@fields = split(/\|/,$line);
					$pid = $fields[1];
					if (exists $h_pid2header{$pid})
					{	$get_seq=1;
						$nof_entries++;
						$h_pid2count{$pid}++;
						print FH_MERGED "$h_pid2header{$pid}\n";
					}
					else
					{	print LOG "unmatched protein:$line\n";  $get_seq=0;}
				}
				else
				{
					if ($get_seq)
					{	print FH_MERGED "$line\n";	}
				}
			}
			close(FH);
			system('rm',$fasta);

		}
	}
}

##############################################
	## PRINT COUNTS
##############################################
	print  "************************************\n";
	print  "$nof_entries total proteins merged! \n";
	print LOG "the following genes had no valid proteins:\n";

	foreach $pid (sort keys %h_pid2count)
	{	if ($h_pid2count{$pid}<1)
		{	print LOG "$pid:$h_pid2header{$pid}\n";	}
	}
close(FH_MERGED);
close(FH_HEADERS);
close(LOG);
